module CES
	# entity class (each separate entity)
	class Entity
		@@entity_list = {}
		
		attr_reader :eid
		
		# components -> Array<String component_name>
		def initialize(*components)
			if(components == nil)
				components = []
			end
		
			@eid = @@entity_list.keys.length + 1 # eids start counting at 1: entity 1, entity 2, entity 3, etc
			@@entity_list[eid] = self
			
			components.each { |component|
				CES::ComponentReference.register(component, @eid)
			}
			
			return self
		end
		
		def self.entity_list
			return @@entity_list
		end
	end
	# component reference - which entities have which components
	class ComponentReference
		@@components = {}
		
		# allow Component.loadcomponents to add components to ComponentReference
		def self.loadedcomponents(components = [])
			components.each { |component|
				@@components[component["meta"]["name"]] = {
					"component" => component,
					"entities" => []
				}
			}
		end
		
		# allow reading of all components
		def self.components
			return @@components
		end
		
		# allow adding of EIDs to component eid list
		# component -> name, eid -> number
		def self.register(component, eid)
			@@components[component]["entities"].push(eid)
		end
		# remove eid from component's list
		def self.unregister(component, eid)
			@@components[component]["entities"].delete(eid)
		end
	end
end